@@ -23,7 +23,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	20 December 1989
- * Last Edited:	30 August 2007
+ * Last Edited:	11 October 2007
  */
 
 
@@ -695,7 +695,7 @@ char *unix_text_work (MAILSTREAM *stream,MESSAGECACHE *elt,
 {
   FDDATA d;
   STRING bs;
-  unsigned char *s,*t,*tl,tmp[CHUNKSIZE];
+  unsigned char c,*s,*t,*tl,tmp[CHUNKSIZE];
 				/* go to text position */
   lseek (LOCAL->fd,elt->private.special.offset +
 	 elt->private.msg.text.offset,L_SET);
@@ -733,13 +733,13 @@ char *unix_text_work (MAILSTREAM *stream,MESSAGECACHE *elt,
     d.chunk = tmp;		/* initial buffer chunk */
     d.chunksize = CHUNKSIZE;	/* file chunk size */
     INIT (&bs,fd_string,&d,elt->private.msg.text.text.size);
-    for (s = (char *) LOCAL->text.data; SIZE (&bs);) switch (CHR (&bs)) {
+    for (s = (char *) LOCAL->text.data; SIZE (&bs);) switch (c = SNX (&bs)) {
     case '\r':			/* carriage return seen */
       break;
     case '\n':
       *s++ = '\r';		/* insert a CR */
     default:
-      *s++ = SNX (&bs);		/* copy characters */
+      *s++ = c;			/* copy characters */
     }
     *s = '\0';			/* tie off buffer */
 				/* calculate length of cached data */
@@ -2640,7 +2640,7 @@ long mbox_ping (MAILSTREAM *stream)
 	else {			/* data copied OK */
 	  ftruncate (sfd,0);	/* truncate sysinbox to zero bytes */
 	  if (!snarfed++) {	/* have we snarfed before? */
-				/* syslog if server, else mm_log() */
+				/* syslog if server, else user log */
 	    sprintf (LOCAL->buf,"Moved %lu bytes of new mail to %s from %s",
 		     size,stream->mailbox,sysinbox ());
 	    if (strcmp ((char *) mail_parameters (NIL,GET_SERVICENAME,NIL),
