@@ -1,5 +1,5 @@
 /* ========================================================================
- * Copyright 1988-2007 University of Washington
+ * Copyright 1988-2008 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,15 +15,13 @@
  * Program:	UNIX mail routines
  *
  * Author:	Mark Crispin
- *		Networks and Distributed Computing
- *		Computing & Communications
+ *		UW Technology
  *		University of Washington
- *		Administration Building, AG-44
  *		Seattle, WA  98195
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	20 December 1989
- * Last Edited:	18 April 2007
+ * Last Edited:	20 February 2008
  */
 
 
@@ -145,7 +143,7 @@ void unix_phys_write (UNIXFILE *f,char *buf,size_t size);
 DRIVER unixdriver = {
   "unix",			/* driver name */
 				/* driver flags */
-  DR_LOCAL|DR_MAIL|DR_NONEWMAILRONLY,
+  DR_LOCAL|DR_MAIL|DR_NONEWMAILRONLY|DR_XPOINT,
   (DRIVER *) NIL,		/* next driver */
   unix_valid,			/* mailbox is valid for us */
   unix_parameters,		/* manipulate parameters */
@@ -459,7 +457,8 @@ MAILSTREAM *unix_open (MAILSTREAM *stream)
       mm_log ("Can't open mailbox lock, access is readonly",WARN);
 				/* can get the lock? */
     else if (flock (fd,LOCK_EX|LOCK_NB)) {
-      mm_log ("Mailbox is open by another process, access is readonly",WARN);
+      if (!stream->silent)
+	mm_log ("Mailbox is open by another process, access is readonly",WARN);
       close (fd);
     }
     else {			/* got the lock, nobody else can alter state */
@@ -628,7 +627,7 @@ char *unix_text_work (MAILSTREAM *stream,MESSAGECACHE *elt,
 {
   FDDATA d;
   STRING bs;
-  unsigned char *s,tmp[CHUNKSIZE];
+  unsigned char c,*s,tmp[CHUNKSIZE];
 				/* go to text position */
   lseek (LOCAL->fd,elt->private.special.offset +
 	 elt->private.msg.text.offset,L_SET);
@@ -660,15 +659,15 @@ char *unix_text_work (MAILSTREAM *stream,MESSAGECACHE *elt,
     d.chunk = tmp;		/* initial buffer chunk */
     d.chunksize = CHUNKSIZE;	/* file chunk size */
     INIT (&bs,fd_string,&d,elt->private.msg.text.text.size);
-    for (s = (char *) LOCAL->text.data; SIZE (&bs);) switch (CHR (&bs)) {
+    for (s = (char *) LOCAL->text.data; SIZE (&bs);) switch (c = SNX (&bs)) {
     case '\r':			/* carriage return seen */
-      *s++ = SNX (&bs);		/* copy it and any succeeding LF */
+      *s++ = c;			/* copy it and any succeeding LF */
       if (SIZE (&bs) && (CHR (&bs) == '\n')) *s++ = SNX (&bs);
       break;
     case '\n':
       *s++ = '\r';		/* insert a CR */
     default:
-      *s++ = SNX (&bs);		/* copy characters */
+      *s++ = c;			/* copy characters */
     }
     *s = '\0';			/* tie off buffer */
 				/* calculate length of cached data */
@@ -824,37 +823,41 @@ long unix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
   if (!((options & CP_UID) ? mail_uid_sequence (stream,sequence) :
 	mail_sequence (stream,sequence))) return NIL;
 				/* make sure destination is valid */
-  if (unix_valid (mailbox) || !errno) {
-				/* try to open rewrite or at least sniff*/
-    if (!(tstream = mail_open_work (&unixdriver,NIL,mailbox,
-				    OP_SILENT|OP_NOKOD)) &&
-	!(tstream = mail_open_work (&unixdriver,NIL,mailbox,
-				    OP_READONLY|OP_SILENT|OP_NOKOD|OP_SNIFF))){
-      sprintf (LOCAL->buf,"Unable to open mailbox for COPY: %.80s",mailbox);
+  if (!(unix_valid (mailbox) || !errno))
+    switch (errno) {
+    case ENOENT:			/* no such file? */
+      if (compare_cstring (mailbox,"INBOX")) {
+	mm_notify (stream,"[TRYCREATE] Must create mailbox before copy",NIL);
+	return NIL;
+      }
+      if (pc) return (*pc) (stream,sequence,mailbox,options);
+      unix_create (NIL,"INBOX");/* create empty INBOX */
+    case EACCES:		/* file protected */
+      sprintf (LOCAL->buf,"Can't access destination: %.80s",mailbox);
       MM_LOG (LOCAL->buf,ERROR);
       return NIL;
-    }
-  }
-  else switch (errno) {
-  case ENOENT:			/* no such file? */
-    if (compare_cstring (mailbox,"INBOX")) {
-      mm_notify (stream,"[TRYCREATE] Must create mailbox before copy",NIL);
+    case EINVAL:
+      if (pc) return (*pc) (stream,sequence,mailbox,options);
+      sprintf (LOCAL->buf,"Invalid UNIX-format mailbox name: %.80s",mailbox);
+      mm_log (LOCAL->buf,ERROR);
+      return NIL;
+    default:
+      if (pc) return (*pc) (stream,sequence,mailbox,options);
+      sprintf (LOCAL->buf,"Not a UNIX-format mailbox: %.80s",mailbox);
+      mm_log (LOCAL->buf,ERROR);
       return NIL;
     }
-    if (pc) return (*pc) (stream,sequence,mailbox,options);
-    unix_create (NIL,"INBOX");/* create empty INBOX */
-  case EINVAL:
-    if (pc) return (*pc) (stream,sequence,mailbox,options);
-    sprintf (LOCAL->buf,"Invalid UNIX-format mailbox name: %.80s",mailbox);
-    mm_log (LOCAL->buf,ERROR);
-    return NIL;
-  default:
-    if (pc) return (*pc) (stream,sequence,mailbox,options);
-    sprintf (LOCAL->buf,"Not a UNIX-format mailbox: %.80s",mailbox);
-    mm_log (LOCAL->buf,ERROR);
-    return NIL;
-  }
 
+				/* try to open rewrite for UIDPLUS */
+  if ((tstream = mail_open_work (&unixdriver,NIL,mailbox,
+				 OP_SILENT|OP_NOKOD)) && tstream->rdonly)
+    tstream = mail_close (tstream);
+  if (cu && !tstream) {		/* wanted a COPYUID? */
+    sprintf (LOCAL->buf,"Unable to write-open mailbox for COPYUID: %.80s",
+	     mailbox);
+    MM_LOG (LOCAL->buf,WARN);
+    cu = NIL;			/* don't try to do COPYUID */
+  }
   LOCAL->buf[0] = '\0';
   mm_critical (stream);		/* go critical */
   if ((fd = unix_lock (dummy_file (file,mailbox),
@@ -882,8 +885,9 @@ long unix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 	if (j && (s[j - 4] == '\r')) j -= 2;
 	if (write (fd,s,j) < 0) ret = NIL;
 	else {			/* message header succeeded */
-	  j = unix_xstatus (stream,LOCAL->buf,elt,
-			    cu ? ++(tstream->uid_last) : NIL,cu ? LONGT : NIL);
+	  j = tstream ?		/* write UIDPLUS data if have readwrite */
+	    unix_xstatus (stream,LOCAL->buf,elt,++(tstream->uid_last),LONGT) :
+	    unix_xstatus (stream,LOCAL->buf,elt,NIL,NIL);
 	  if (write (fd,LOCAL->buf,j) < 0) ret = NIL;
 	  else {		/* message status succeeded */
 	    s = unix_text_work (stream,elt,&j,NIL);
@@ -904,12 +908,10 @@ long unix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
     ret = NIL;
   }
 				/* force UIDVALIDITY assignment now */
-  if (!tstream->uid_validity && !tstream->rdonly)
+  if (tstream && !tstream->uid_validity)
     tstream->uid_validity = (unsigned long) time (0);
-
 				/* return sets if doing COPYUID */
-  if (cu && ret && tstream->uid_validity)
-    (*cu) (stream,mailbox,tstream->uid_validity,source,dest);
+  if (cu && ret) (*cu) (stream,mailbox,tstream->uid_validity,source,dest);
   else {			/* flush any sets we may have built */
     mail_free_searchset (&source);
     mail_free_searchset (&dest);
@@ -923,12 +925,12 @@ long unix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 	 sbuf.st_atime : times.modtime;
   utime (file,&times);		/* set the times */
   unix_unlock (fd,NIL,lock);	/* unlock and close mailbox */
-  if (!tstream->rdonly) {	/* update last UID if we can */
+  if (tstream) {		/* update last UID if we can */
     UNIXLOCAL * local = (UNIXLOCAL *) tstream->local;
     local->dirty = T;		/* do a rewrite */
     local->appending = T;	/* but not at the cost of marking as old */
+    tstream = mail_close (tstream);
   }
-  tstream = mail_close (tstream);
 				/* log the error */
   if (!ret) mm_log (LOCAL->buf,ERROR);
 				/* delete if requested message */
@@ -971,18 +973,8 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     stream = &unixproto;	/* no, default stream to prototype */
     for (i = 0; i < NUSERFLAGS && stream->user_flags[i]; ++i)
       fs_give ((void **) &stream->user_flags[i]);
-    stream->kwd_create = T;	/* allow new flags */
   }
-  if (unix_valid (mailbox)) {	/* make sure valid mailbox */
-    if (!(tstream = mail_open (NIL,mailbox,
-			       OP_READONLY|OP_SILENT|OP_NOKOD|OP_SNIFF))) {
-      sprintf (tmp,"Unable to examine mailbox for APPEND: %.80s",mailbox);
-      MM_LOG (tmp,ERROR);
-      return NIL;
-    }
-  }
-
-  else switch (errno) {
+  if (!unix_valid (mailbox)) switch (errno) {
   case ENOENT:			/* no such file? */
     if (!compare_cstring (mailbox,"INBOX")) {
       mm_notify (stream,"[TRYCREATE] Must create mailbox before append",NIL);
@@ -992,6 +984,10 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
   case 0:			/* merely empty file? */
     tstream = stream;
     break;
+  case EACCES:			/* file protected */
+    sprintf (tmp,"Can't access destination: %.80s",mailbox);
+    MM_LOG (tmp,ERROR);
+    return NIL;
   case EINVAL:
     sprintf (tmp,"Invalid UNIX-format mailbox name: %.80s",mailbox);
     mm_log (tmp,ERROR);
@@ -1001,6 +997,14 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     mm_log (tmp,ERROR);
     return NIL;
   }
+				/* get sniffing stream for keywords */
+  else if (!(tstream = mail_open (NIL,mailbox,
+				  OP_READONLY|OP_SILENT|OP_NOKOD|OP_SNIFF))) {
+    sprintf (tmp,"Unable to examine mailbox for APPEND: %.80s",mailbox);
+    MM_LOG (tmp,ERROR);
+    return NIL;
+  }
+
 				/* get first message */
   if (!(*af) (tstream,data,&flags,&date,&message)) return NIL;
   if (!(sf = tmpfile ())) {	/* must have scratch file */
@@ -1012,7 +1016,6 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     }
     unlink (tmp);
   }
-
   do {				/* parse date */
     if (!date) rfc822_date (date = tmp);
     if (!mail_parse_date (&elt,date)) {
@@ -1037,26 +1040,25 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     fclose (sf);		/* punt scratch file */
     return NIL;			/* give up */
   } while (message);		/* until no more messages */
-  if (fflush (sf) || fstat (fileno (sf),&sbuf)) {
+  if (fflush (sf)) {
     sprintf (tmp,"Error finishing scratch file: %.80s",strerror (errno));
     mm_log (tmp,ERROR);
     fclose (sf);		/* punt scratch file */
     return NIL;			/* give up */
   }
-  i = sbuf.st_size;		/* size of scratch file */
+  i = ftell (sf);		/* size of scratch file */
 
 				/* close sniffing stream */
   if (tstream != stream) tstream = mail_close (tstream);
   mm_critical (stream);		/* go critical */
-				/* try to open readwrite */
-  if (!(tstream = mail_open_work (&unixdriver,NIL,mailbox,
-				  OP_SILENT|OP_NOKOD)) &&
-				/* damn, someone else has it open */
-      !(tstream = mail_open_work (&unixdriver,NIL,mailbox,
-				  OP_READONLY|OP_SILENT|OP_NOKOD|OP_SNIFF))) {
-    sprintf (tmp,"Unable to re-open mailbox for APPEND: %.80s",mailbox);
-    MM_LOG (tmp,ERROR);
-    return NIL;
+				/* try to open readwrite for UIDPLUS */
+  if ((tstream = mail_open_work (&unixdriver,NIL,mailbox,
+				 OP_SILENT|OP_NOKOD)) && tstream->rdonly)
+    tstream = mail_close (tstream);
+  if (au && !tstream) {		/* wanted an APPENDUID? */
+    sprintf (tmp,"Unable to re-open mailbox for APPENDUID: %.80s",mailbox);
+    MM_LOG (tmp,WARN);
+    au = NIL;
   }
   if (((fd = unix_lock (dummy_file (file,mailbox),
 			O_BINARY|O_WRONLY|O_APPEND|O_CREAT,S_IREAD|S_IWRITE,
@@ -1085,21 +1087,20 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
   utime (file,&times);		/* set the times */
   fclose (sf);			/* done with scratch file */
 				/* force UIDVALIDITY assignment now */
-  if (!tstream->uid_validity && !tstream->rdonly)
+  if (tstream && !tstream->uid_validity)
     tstream->uid_validity = (unsigned long) time (0);
 				/* return sets if doing APPENDUID */
-  if (au && ret && tstream->uid_validity)
-    (*au) (mailbox,tstream->uid_validity,dst);
+  if (au && ret) (*au) (mailbox,tstream->uid_validity,dst);
   else mail_free_searchset (&dst);
   flock (fd,LOCK_UN);		/* unlock mailbox (can't use unix_unlock() */
   if (lock && *lock) unlink (lock);
   fclose (df);			/* close mailbox */
-  if (!tstream->rdonly) {	/* update last UID if we can */
+  if (tstream) {		/* update last UID if we can */
     UNIXLOCAL * local = (UNIXLOCAL *) tstream->local;
     local->dirty = T;		/* do a rewrite */
     local->appending = T;	/* but not at the cost of marking as old */
+    tstream = mail_close (tstream);
   }
-  tstream = mail_close (tstream);
   mm_nocritical (stream);	/* release critical */
   return ret;
 }
@@ -1108,6 +1109,7 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
  * Accepts: MAIL stream
  *	    scratch file
  *	    flags
+ *	    date
  *	    message stringstruct
  * Returns: NIL if write error, else T
  */
@@ -1118,8 +1120,19 @@ int unix_collect_msg (MAILSTREAM *stream,FILE *sf,char *flags,char *date,
   unsigned char *s,*t;
   unsigned long uf;
   long f = mail_parse_flags (stream,flags,&uf);
-				/* write metadata, note date ends with NL */
-  if (fprintf (sf,"%ld %lu %lu %s",f,uf,SIZE (msg) + 1,date) < 0) return NIL;
+				/* write metadata */
+  if (fprintf (sf,"%ld %lu ",f,SIZE (msg) + 2) < 0) return NIL;
+  for (s = date; *s; *s++) switch (*s) {
+  default:
+    if (putc (*s,sf) == EOF) return NIL;
+  case '\r': case '\n':
+    break;
+  }
+  if (fputs ("\r\n",sf) == EOF) return NIL;
+  while (uf)			/* write user flags */    
+    if ((s = stream->user_flags[find_rightmost_bit (&uf)]) &&
+	(fprintf (sf," %s",s) < 0)) return NIL;
+  if (fputs ("\r\n",sf) == EOF) return NIL;
   while (SIZE (msg)) {		/* copy text to scratch file */
     for (s = (unsigned char *) msg->curpos, t = s + msg->cursize; s < t; ++s)
       if (!*s) *s = 0x80;	/* disallow NUL */
@@ -1128,8 +1141,8 @@ int unix_collect_msg (MAILSTREAM *stream,FILE *sf,char *flags,char *date,
       SETPOS (msg,GETPOS (msg) + msg->cursize);
     else return NIL;		/* failed */
   }
-				/* write trailing newline and return */
-  return (putc ('\n',sf) == EOF) ? NIL : T;
+				/* write trailing CRLF and return */
+  return (fputs ("\r\n",sf) == EOF) ? NIL : T;
 }
 
 /* Append messages from scratch file to mailbox
@@ -1144,7 +1157,7 @@ int unix_append_msgs (MAILSTREAM *stream,FILE *sf,FILE *df,SEARCHSET *set)
 {
   int ti,zn,c;
   long f;
-  unsigned long i,j,uf;
+  unsigned long i,j;
   char *x,tmp[MAILTMPLEN];
   int hdrp = T;
 				/* get message metadata line */
@@ -1152,25 +1165,26 @@ int unix_append_msgs (MAILSTREAM *stream,FILE *sf,FILE *df,SEARCHSET *set)
     if (!(isdigit (tmp[0]) && strchr (tmp,'\n'))) return NIL;
     f = strtol (tmp,&x,10);	/* get flags */
     if (!((*x++ == ' ') && isdigit (*x))) return NIL;
-    uf = strtoul (x,&x,10);	/* get keywords */
-    if (!((*x++ == ' ') && isdigit (*x))) return NIL;
     i = strtoul (x,&x,10);	/* get message size */
     if ((*x++ != ' ') ||	/* build initial header */
 	(fprintf (df,"From %s@%s %sStatus: ",myusername(),mylocalhost(),x)<0)||
 	(f&fSEEN && (putc ('R',df) == EOF)) ||
-	(fputs ("\nX-Status: ",df) == EOF) ||
+	(fputs ("\r\nX-Status: ",df) == EOF) ||
 	(f&fDELETED && (putc ('D',df) == EOF)) ||
 	(f&fFLAGGED && (putc ('F',df) == EOF)) ||
 	(f&fANSWERED && (putc ('A',df) == EOF)) ||
 	(f&fDRAFT && (putc ('T',df) == EOF)) ||
-	(fputs ("\nX-Keywords:",df) == EOF)) return NIL;
-    while (uf)			/* write user flags */
-      if (fprintf (df," %s",stream->user_flags[find_rightmost_bit (&uf)]) < 0)
-	return NIL;
-    if (set && (fprintf (df,"\nX-UID: %lu",++(stream->uid_last)) < 0))
+	(fputs ("\r\nX-Keywords:",df) == EOF)) return NIL;
+				/* copy keywords */
+    while ((c = getc (sf)) != '\n') switch (c) {
+    case EOF:
+      return NIL;
+    default:
+      if (putc (c,df) == EOF) return NIL;
+    }
+    if ((putc ('\n',df) == EOF) ||
+	(set && (fprintf (df,"X-UID: %lu\r\n",++(stream->uid_last)) < 0)))
       return NIL;
-				/* finish last metadata header */
-    if (putc ('\n',df) == EOF) return NIL;
 
     for (c = '\n'; i && fgets (tmp,MAILTMPLEN,sf); c = tmp[j-1]) {
 				/* get read line length */
@@ -1180,17 +1194,17 @@ int unix_append_msgs (MAILSTREAM *stream,FILE *sf,FILE *df,SEARCHSET *set)
 				/* complete line? */
       if ((c == '\n')) switch (tmp[0]) {
       case 'F':			/* possible "From " (case counts here) */
-	if ((i > 4) && (tmp[0] == 'F') && (tmp[1] == 'r') && (tmp[2] == 'o') &&
+	if ((j > 4) && (tmp[0] == 'F') && (tmp[1] == 'r') && (tmp[2] == 'o') &&
 	    (tmp[3] == 'm') && (tmp[4] == ' ')) {
 	  if (!unix_fromwidget) {
 	    VALID (tmp,x,ti,zn);/* conditional, only write widget if */
 	    if (!ti) break;	/*  it looks like a valid header */
-	  }				/* write the widget */
+	  }			/* write the widget */
 	  if (putc ('>',df) == EOF) return NIL;
 	}
 	break;
       case 'S': case 's':	/* possible "Status:" */
-	if (hdrp && (i > 6) && ((tmp[1] == 't') || (tmp[1] == 'T')) &&
+	if (hdrp && (j > 6) && ((tmp[1] == 't') || (tmp[1] == 'T')) &&
 	    ((tmp[2] == 'a') || (tmp[2] == 'A')) &&
 	    ((tmp[3] == 't') || (tmp[3] == 'T')) &&
 	    ((tmp[4] == 'u') || (tmp[4] == 'U')) &&
@@ -1200,29 +1214,29 @@ int unix_append_msgs (MAILSTREAM *stream,FILE *sf,FILE *df,SEARCHSET *set)
       case 'X': case 'x':	/* possible X-??? header */
 	if (hdrp && (tmp[1] == '-') &&
 				/* possible X-UID: */
-	    (((i > 5) && ((tmp[2] == 'U') || (tmp[2] == 'u')) &&
+	    (((j > 5) && ((tmp[2] == 'U') || (tmp[2] == 'u')) &&
 	      ((tmp[3] == 'I') || (tmp[3] == 'i')) &&
 	      ((tmp[4] == 'D') || (tmp[4] == 'd')) && (tmp[5] == ':')) ||
 				/* possible X-IMAP: */
-	     ((i > 6) && ((tmp[2] == 'I') || (tmp[2] == 'i')) &&
+	     ((j > 6) && ((tmp[2] == 'I') || (tmp[2] == 'i')) &&
 	      ((tmp[3] == 'M') || (tmp[3] == 'm')) &&
 	      ((tmp[4] == 'A') || (tmp[4] == 'a')) &&
 	      ((tmp[5] == 'P') || (tmp[5] == 'p')) &&
 	      ((tmp[6] == ':') ||
 				/* or X-IMAPbase: */
-	       ((i > 10) && ((tmp[6] == 'b') || (tmp[6] == 'B')) &&
+	       ((j > 10) && ((tmp[6] == 'b') || (tmp[6] == 'B')) &&
 		((tmp[7] == 'a') || (tmp[7] == 'A')) &&
 		((tmp[8] == 's') || (tmp[8] == 'S')) &&
 		((tmp[9] == 'e') || (tmp[9] == 'E')) && (tmp[10] == ':')))) ||
 				/* possible X-Status: */
-	     ((i > 8) && ((tmp[2] == 'S') || (tmp[2] == 's')) &&
+	     ((j > 8) && ((tmp[2] == 'S') || (tmp[2] == 's')) &&
 	      ((tmp[3] == 't') || (tmp[3] == 'T')) &&
 	      ((tmp[4] == 'a') || (tmp[4] == 'A')) &&
 	      ((tmp[5] == 't') || (tmp[5] == 'T')) &&
 	      ((tmp[6] == 'u') || (tmp[6] == 'U')) &&
 	      ((tmp[7] == 's') || (tmp[7] == 'S')) && (tmp[8] == ':')) ||
 				/* possible X-Keywords: */
-	     ((i > 10) && ((tmp[2] == 'K') || (tmp[2] == 'k')) &&
+	     ((j > 10) && ((tmp[2] == 'K') || (tmp[2] == 'k')) &&
 	      ((tmp[3] == 'e') || (tmp[3] == 'E')) &&
 	      ((tmp[4] == 'y') || (tmp[4] == 'Y')) &&
 	      ((tmp[5] == 'w') || (tmp[5] == 'W')) &&
@@ -1243,7 +1257,7 @@ int unix_append_msgs (MAILSTREAM *stream,FILE *sf,FILE *df,SEARCHSET *set)
     }
     if (i) return NIL;		/* didn't read entire message */
 				/* update set */
-    mail_append_set (set,stream->uid_last);
+    if (stream) mail_append_set (set,stream->uid_last);
   }
   return T;
 }
@@ -1671,7 +1685,7 @@ int unix_parse (MAILSTREAM *stream,char *lock,int op)
 	      for (u = s,v = tmp,e = u + min (i,MAILTMPLEN - 1);
 		   (u < e) && ((c = (*u ? *u : (*u = ' '))) != ':') &&
 		   ((c > ' ') || ((c != ' ') && (c != '\t') &&
-				  (c != '\015') && (c != '\012')));
+				  (c != '\r') && (c != '\n')));
 		   *v++ = *u++);
 	      *v = '\0';	/* tie off */
 				/* matches internal header? */
@@ -1694,10 +1708,8 @@ int unix_parse (MAILSTREAM *stream,char *lock,int op)
 	      retain = T;	/* retaining continuation now */
 				/* line length in CRLF format newline */
 	      k = i + (((i < 2) || (s[i - 2] != '\r')) ? 1 : 0);
-				/* "internal" header size */
-	      elt->private.spare.data += k;
-				/* message size */
-	      elt->rfc822_size += k;
+				/* header size */
+	      elt->rfc822_size = elt->private.spare.data += k;
 	    }
 	    else {
 	      char err[MAILTMPLEN];
@@ -1716,6 +1728,7 @@ int unix_parse (MAILSTREAM *stream,char *lock,int op)
 	if (((nmsgs > 1) || !pseudoseen) && !elt->private.uid) {
 	  prevuid = elt->private.uid = ++stream->uid_last;
 	  elt->private.dirty = T;
+	  LOCAL->ddirty = T;	/* force update */
 	}
 	else elt->private.dirty = elt->recent;
 
@@ -1862,7 +1875,6 @@ char *unix_mbxline (MAILSTREAM *stream,STRING *bs,unsigned long *size)
       if (!bs->cursize) SETPOS (bs,GETPOS (bs));
 				/* read newline at end */
       if (SIZE (bs)) ret[i++] = SNX (bs);
-      ret[i++] = '\n';		/* make sure newline at end */
       ret[i] = '\0';		/* makes debugging easier */
     }
     else {			/* this is easy */
