@@ -1,5 +1,5 @@
 /* ========================================================================
- * Copyright 1988-2007 University of Washington
+ * Copyright 1988-2008 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,15 +15,13 @@
  * Program:	MIX mail routines
  *
  * Author(s):	Mark Crispin
- *		Networks and Distributed Computing
- *		Computing & Communications
+ *		UW Technology
  *		University of Washington
- *		Administration Building, AG-44
  *		Seattle, WA  98195
- *		Internet: MRC@CAC.Washington.EDU
+ *		Internet: MRC@Washington.EDU
  *
  * Date:	1 March 2006
- * Last Edited:	31 July 2007
+ * Last Edited:	19 February 2008
  */
 
 
@@ -73,12 +71,6 @@ extern int errno;		/* just in case */
 #define MSGTSZ (sizeof(MSGTOK)-1)
 				/* sortcache file record format */
 #define SCRFMT ":%08lx:%08lx:%08lx:%08lx:%08lx:%c%08lx:%08lx:%08lx:\015\012"
-
-
-/* mix_parse() flags */
-
-#define MP_WRITE 0x1		/* open for write */
-#define MP_VALID 0x2		/* check in recent messages (implies write) */
 
 /* MIX I/O stream local data */
 	
@@ -158,17 +150,17 @@ long mix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data);
 long mix_append_msg (MAILSTREAM *stream,FILE *f,char *flags,MESSAGECACHE *delt,
 		     STRING *msg,SEARCHSET *set,unsigned long seq);
 
-FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags);
+FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long sflags);
 char *mix_meta_slurp (MAILSTREAM *stream,unsigned long *seq);
 long mix_meta_update (MAILSTREAM *stream);
 long mix_index_update (MAILSTREAM *stream,FILE *idxf);
-long mix_status_update (MAILSTREAM *stream,FILE **statf,long flag);
+long mix_status_update (MAILSTREAM *stream,FILE *statf);
 FILE *mix_data_open (MAILSTREAM *stream,int *fd,long *size,
 		     unsigned long newsize);
 FILE *mix_sortcache_open (MAILSTREAM *stream);
 long mix_sortcache_update (MAILSTREAM *stream,FILE **sortcache);
 char *mix_read_record (FILE *f,char *buf,unsigned long buflen,char *type);
-long mix_read_sequence (FILE *f);
+unsigned long mix_read_sequence (FILE *f);
 char *mix_dir (char *dst,char *name);
 char *mix_file (char *dst,char *dir,char *name);
 char *mix_file_data (char *dst,char *dir,unsigned long data);
@@ -673,7 +665,7 @@ void mix_close (MAILSTREAM *stream,long options)
   if (LOCAL) {			/* only if a file is open */
     int silent = stream->silent;
     stream->silent = T;		/* note this stream is dying */
-				/* checkpoint or expunge */
+				/* burp-only or expunge */
     mix_expunge (stream,(options & CL_EXPUNGE) ? NIL : "",NIL);
 				/* close current message file if open */
     if (LOCAL->msgfd >= 0) close (LOCAL->msgfd);
@@ -738,7 +730,7 @@ char *mix_header (MAILSTREAM *stream,unsigned long msgno,unsigned long *length,
     i = elt->private.msg.header.offset;
     *length = elt->private.msg.header.text.size;
     if (k != elt->rfc822_size) {
-      sprintf (tmp,"Inconsistency in mix message size, uid=%.08x (%lu != %lu)",
+      sprintf (tmp,"Inconsistency in mix message size, uid=%lx (%lu != %lu)",
 	       elt->private.uid,elt->rfc822_size,k);
       MM_LOG (tmp,WARN);
     }
@@ -747,7 +739,7 @@ char *mix_header (MAILSTREAM *stream,unsigned long msgno,unsigned long *length,
     LOCAL->buf[100] = '\0';	/* tie off buffer at no more than 100 octets */
 				/* or at newline, whichever is first */
     if (s = strpbrk (LOCAL->buf,"\015\012")) *s = '\0';
-    sprintf (tmp,"Error reading mix message header, u=%.08x, s=%.0lx, h=%s",
+    sprintf (tmp,"Error reading mix message header, uid=%lx, s=%.0lx, h=%s",
 	     elt->private.uid,elt->rfc822_size,LOCAL->buf);
     MM_LOG (tmp,ERROR);
     *length = i = j = 0;	/* default to empty */
@@ -784,15 +776,16 @@ long mix_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags)
 				/* doing non-peek fetch? */
   if (!(flags & FT_PEEK) && !elt->seen) {
     FILE *idxf;			/* yes, process metadata/index/status */
-    FILE *statf = mix_parse (stream,&idxf,NIL,stream->rdonly ? NIL : MP_VALID);
+    FILE *statf = mix_parse (stream,&idxf,NIL,LONGT);
     elt->seen = T;		/* mark as seen */
     MM_FLAGS (stream,elt->msgno);
-    if (statf) {		/* update status file if possible */
-      if (!stream->rdonly)
-	elt->private.mod = LOCAL->statusseq = mix_modseq (LOCAL->statusseq);
-      mix_status_update (stream,&statf,LONGT);
+				/* update status file if possible */
+    if (statf && !stream->rdonly) {
+      elt->private.mod = LOCAL->statusseq = mix_modseq (LOCAL->statusseq);
+      mix_status_update (stream,statf);
     }
-    if (idxf) fclose (idxf);	/* release index file */
+    if (idxf) fclose (idxf);	/* release index and status file */
+    if (statf) fclose (statf);
   } 
   d.fd = LOCAL->msgfd;		/* set up file descriptor */
 				/* offset of message text */
@@ -818,10 +811,10 @@ void mix_flag (MAILSTREAM *stream,char *sequence,char *flag,long flags)
   long f;
   short nf;
   FILE *idxf;
-  FILE *statf = mix_parse (stream,&idxf,NIL,stream->rdonly ? NIL : MP_VALID);
+  FILE *statf = mix_parse (stream,&idxf,NIL,LONGT);
   unsigned long seq = mix_modseq (LOCAL->statusseq);
 				/* find first free key */
-  for (ffkey = 0; (ffkey < NUSERFLAGS) && stream->user_flags[ffkey];++ffkey);
+  for (ffkey = 0; (ffkey < NUSERFLAGS) && stream->user_flags[ffkey]; ++ffkey);
 				/* parse sequence and flags */
   if (((flags & ST_UID) ? mail_uid_sequence (stream,sequence) :
        mail_sequence (stream,sequence)) &&
@@ -856,13 +849,12 @@ void mix_flag (MAILSTREAM *stream,char *sequence,char *flag,long flags)
 	  MM_FLAGS (stream,elt->msgno);
 	}
       }
-    if (statf) {		/* update and close status file after change */
-      if (seq == LOCAL->statusseq) mix_status_update (stream,&statf,LONGT);
+				/* update status file after change */
+    if (statf && (seq == LOCAL->statusseq)) mix_status_update (stream,statf);
 				/* update metadata if created a keyword */
-      if ((ffkey < NUSERFLAGS) && stream->user_flags[ffkey] &&
-	  !mix_meta_update (stream))
-	MM_LOG ("Error updating mix metadata after keyword creation",ERROR);
-    }
+    if ((ffkey < NUSERFLAGS) && stream->user_flags[ffkey] &&
+	!mix_meta_update (stream))
+      MM_LOG ("Error updating mix metadata after keyword creation",ERROR);
   }
   if (statf) fclose (statf);	/* release status file if still open */
   if (idxf) fclose (idxf);	/* release index file */
@@ -981,9 +973,8 @@ long mix_ping (MAILSTREAM *stream)
 				/* expunging OK if global flag set */
   if (mail_parameters (NIL,GET_EXPUNGEATPING,NIL)) LOCAL->expok = T;
 				/* process metadata/index/status */
-  if (statf = mix_parse (stream,&idxf,stream->rdonly ? NIL : LONGT,
-			 stream->rdonly ? NIL :
-			 (LOCAL->internal ? MP_WRITE : MP_VALID))) {
+  if (statf = mix_parse (stream,&idxf,LONGT,
+			 (LOCAL->internal ? NIL : LONGT))) {
     fclose (statf);		/* just close the status file */
     ret = LONGT;		/* declare success */
   }
@@ -995,7 +986,7 @@ long mix_ping (MAILSTREAM *stream)
 }
 
 
-/* MIX mail check mailbox
+/* MIX mail checkpoint mailbox (burp only)
  * Accepts: MAIL stream
  */
 
@@ -1003,37 +994,38 @@ void mix_check (MAILSTREAM *stream)
 {
   if (stream->rdonly)		/* won't do on readonly files! */
     MM_LOG ("Checkpoint ignored on readonly mailbox",NIL);
-				/* do an expunge of no messages */
+				/* do burp-only expunge action */
   if (mix_expunge (stream,"",NIL)) MM_LOG ("Check completed",(long) NIL);
 }
 
 /* MIX mail expunge mailbox
  * Accepts: MAIL stream
- *	    sequence to expunge if non-NIL
+ *	    sequence to expunge if non-NIL, empty string for burp only
  *	    expunge options
  * Returns: T on success, NIL if failure
  */
 
 long mix_expunge (MAILSTREAM *stream,char *sequence,long options)
 {
-  FILE *statf;
   FILE *idxf = NIL;
+  FILE *statf = NIL;
   MESSAGECACHE *elt;
   int ifd,sfd;
   long ret;
   unsigned long i;
   unsigned long nexp = 0;
   unsigned long reclaimed = 0;
-  int checkpoint = (sequence && !*sequence);
+  int burponly = (sequence && !*sequence);
   LOCAL->expok = T;		/* expunge during ping is OK */
-  if (!(ret = checkpoint || !sequence ||
+  if (!(ret = burponly || !sequence ||
 	((options & EX_UID) ?
 	 mail_uid_sequence (stream,sequence) :
-	 mail_sequence (stream,sequence))));
-  else if (!stream->rdonly &&	/* read index and open status exclusive */
-	   (statf = mix_parse (stream,&idxf,LONGT,MP_VALID))) {
-				/* expunge unless just checkpointing */
-    if (!checkpoint) for (i = 1; i <= stream->nmsgs;) {
+	 mail_sequence (stream,sequence))) || stream->rdonly);
+				/* read index and open status exclusive */
+  else if (statf = mix_parse (stream,&idxf,LONGT,
+			      LOCAL->internal ? NIL : LONGT)) {
+				/* expunge unless just burping */
+    if (!burponly) for (i = 1; i <= stream->nmsgs;) {
       elt = mail_elt (stream,i);/* need to expunge this message? */
       if (sequence ? elt->sequence : elt->deleted) {
 	++nexp;			/* yes, make it so */
@@ -1079,7 +1071,7 @@ long mix_expunge (MAILSTREAM *stream,char *sequence,long options)
 				     elt->private.msg.header.offset +
 				     elt->rfc822_size);
 	  else {		/* uh-oh */
-	    sprintf (LOCAL->buf,"Can't locate mix message file %.08x",
+	    sprintf (LOCAL->buf,"Can't locate mix message file %.08lx",
 		     elt->private.spare.data);
 	    MM_LOG (LOCAL->buf,ERROR);
 	    ret = NIL;
@@ -1114,21 +1106,21 @@ long mix_expunge (MAILSTREAM *stream,char *sequence,long options)
       if (mix_index_update (stream,idxf)) {
 	LOCAL->statusseq = mix_modseq (LOCAL->statusseq);
 				/* set failure if update fails */
-	ret = mix_status_update (stream,&statf,LONGT);
+	ret = mix_status_update (stream,statf);
       }
     }
-    if (statf) fclose (statf);	/* close status if still open */
   }
+  if (statf) fclose (statf);	/* close status if still open */
   if (idxf) fclose (idxf);	/* close index if still open */
   LOCAL->expok = NIL;		/* cancel expok */
   if (ret) {			/* only if success */
-    char *s;
+    char *s = NIL;
     if (nexp) sprintf (s = LOCAL->buf,"Expunged %lu messages",nexp);
     else if (reclaimed)
       sprintf (s=LOCAL->buf,"Reclaimed %lu bytes of expunged space",reclaimed);
-    else s = checkpoint ? NIL :
-      (stream->rdonly ? "Expunge ignored on readonly mailbox" :
-       "No messages deleted, so no update needed");
+    else if (!burponly)
+      s = stream->rdonly ? "Expunge ignored on readonly mailbox" :
+	   "No messages deleted, so no update needed";
     if (s) MM_LOG (s,(long) NIL);
   }
   return ret;
@@ -1371,7 +1363,9 @@ long mix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 		    mail_sequence (stream,sequence))));
 				/* acquire stream to append */
   else if (ret = ((astream = mail_open (NIL,mailbox,OP_SILENT)) &&
-		  (statf = mix_parse (astream,&idxf,LONGT,MP_WRITE))) ?
+		  !astream->rdonly &&
+		  (((MIXLOCAL *) astream->local)->expok = T) &&
+		  (statf = mix_parse (astream,&idxf,LONGT,NIL))) ?
 	   LONGT : NIL) {
     int fd;
     unsigned long i;
@@ -1443,8 +1437,8 @@ long mix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 				/* finish write if success */
       if (ret && (ret = !fflush (msgf))) {
 	fclose (msgf);		/* all good, close the msg file now */
-	if (!stream->rdonly)	/* write new metadata, index, and status */
-	  local->metaseq = local->indexseq = local->statusseq = seq;
+				/* write new metadata, index, and status */
+	local->metaseq = local->indexseq = local->statusseq = seq;
 	if (ret = (mix_meta_update (astream) &&
 		   mix_index_update (astream,idxf))) {
 				/* success, delete if doing a move */
@@ -1456,7 +1450,7 @@ long mix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 		MM_FLAGS (stream,elt->msgno);
 	      }
 				/* done with status file now */
-	  mix_status_update (astream,&statf,LONGT);
+	  mix_status_update (astream,statf);
 				/* return sets if doing COPYUID */
 	  if (cu) (*cu) (stream,mailbox,astream->uid_validity,source,dest);
 	  source = dest = NIL;	/* don't free these sets now */
@@ -1527,7 +1521,9 @@ long mix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     FILE *msgf = NIL;
     FILE *statf = NIL;
     if (ret = ((astream = mail_open (NIL,mailbox,OP_SILENT)) &&
-	       (statf = mix_parse (astream,&idxf,LONGT,MP_WRITE))) ?
+	       !astream->rdonly &&
+	       (((MIXLOCAL *) astream->local)->expok = T) &&
+	       (statf = mix_parse (astream,&idxf,LONGT,NIL))) ?
 	LONGT : NIL) {
       int fd;
       unsigned long size,hdrsize;
@@ -1568,11 +1564,11 @@ long mix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
 				/* finish write if success */
 	if (ret && (ret = !fflush (msgf))) {
 	  fclose (msgf);	/* all good, close the msg file now */
-	  if (!stream->rdonly)	/* write new metadata, index, and status */
-	    local->metaseq = local->indexseq = local->statusseq = seq;
+				/* write new metadata, index, and status */
+	  local->metaseq = local->indexseq = local->statusseq = seq;
 	  if ((ret = (mix_meta_update (astream) &&
 		      mix_index_update (astream,idxf) &&
-		      mix_status_update (astream,&statf,LONGT))) && au) {
+		      mix_status_update (astream,statf))) && au) {
 	      (*au) (mailbox,astream->uid_validity,dst);
 	      dst = NIL;	/* don't free this set now */
 	  }
@@ -1697,7 +1693,7 @@ long mix_append_msg (MAILSTREAM *stream,FILE *f,char *flags,MESSAGECACHE *delt,
  * Accepts: MAIL stream
  *	    returned index file
  *	    index file flags (non-NIL if want to add/remove messages)
- *	    status file flags (MP_WRITE, MP_VALID, or NIL)
+ *	    status file flags (non-NIL if want to update elt->valid and old)
  * Returns: open status file, or NIL if failure
  *
  * Note that this routine can return an open index file even if it fails!
@@ -1706,10 +1702,10 @@ long mix_append_msg (MAILSTREAM *stream,FILE *f,char *flags,MESSAGECACHE *delt,
 static char *shortmsg =
   "message %lu (UID=%.08lx) truncated by %lu byte(s) (%lu < %lu)";
 
-FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
+FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long sflags)
 {
   int fd;
-  unsigned long i,j;
+  unsigned long i;
   char *s,*t;
   struct stat sbuf;
   FILE *statf = NIL;
@@ -1717,6 +1713,8 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
   short indexrepairneeded = 0;
   short silent = stream->silent;
   *idxf = NIL;			/* in case error */
+				/* readonly means no updates */
+  if (stream->rdonly) iflags = sflags = NIL;
 				/* open index file */
   if ((fd = open (LOCAL->index,iflags ? O_RDWR : O_RDONLY,NIL)) < 0)
     MM_LOG ("Error opening mix index file",ERROR);
@@ -1730,10 +1728,10 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 
 				/* slurp metadata */
   else if (s = mix_meta_slurp (stream,&i)) {
+    unsigned long j = 0;	/* non-zero if UIDVALIDITY/UIDLAST changed */
     if (i != LOCAL->metaseq) {	/* metadata changed? */
       char *t,*k;
       LOCAL->metaseq = i;	/* note new metadata sequence */
-      j = 0;			/* non-zero if UIDVALIDITY/UIDLAST changes */
       while (s && *s) {		/* parse entire metadata file */
 				/* locate end of line */
 	if (s = strstr (t = s,"\015\012")) {
@@ -1801,7 +1799,9 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
       char *t,*msg,tmp[MAILTMPLEN];
 				/* start with no messages */
       curfile = curfilesize = curpos = nmsgs = 0;
-      LOCAL->indexseq = i;	/* update sequence, get first elt */
+				/* update sequence iff expunging OK */
+      if (LOCAL->expok) LOCAL->indexseq = i;
+				/* get first elt */
       while ((s = mix_read_record (*idxf,LOCAL->buf,LOCAL->buflen,"index")) &&
 	     *s)
 	switch (*s) {
@@ -1859,6 +1859,14 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 			elt = mail_elt (stream,nmsgs);
 				/* existing message with matching data? */
 			if (uid == elt->private.uid) {
+				/* beware of Dracula's ressurection */
+			  if (elt->private.ghost) {
+			    sprintf (tmp,"mix index data unexpunged UID: %lx",
+				     uid);
+			    MM_LOG (tmp,ERROR);
+			    return NIL;
+			  }
+				/* also of static data changing */
 			  if ((size != elt->rfc822_size) ||
 			      (file != elt->private.spare.data) ||
 			      (pos != elt->private.special.offset) ||
@@ -1869,8 +1877,7 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 			      (mm != elt->minutes) || (ss != elt->seconds) ||
 			      (z != elt->zoccident) || (zh != elt->zhours) ||
 			      (zm != elt->zminutes)) {
-			    sprintf (tmp,"mix index data mismatch: %lx",
-				     uid,elt->private.uid);
+			    sprintf (tmp,"mix index data mismatch: %lx",uid);
 			    MM_LOG (tmp,ERROR);
 			    return NIL;
 			  }
@@ -1879,15 +1886,19 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 				/* existing msg with lower UID is expunged */
 			else if (uid > elt->private.uid) {
 			  if (LOCAL->expok) mail_expunged (stream,nmsgs);
-			  else ++nmsgs;
+			  else {/* message expunged, but not yet for us */
+			    ++nmsgs;
+			    elt->private.ghost = T;
+			  }
 			}
 			else {	/* unexpected message record */
-			  sprintf (tmp,"mix index UID mismatch (%lx != %lx)",
+			  sprintf (tmp,"mix index UID mismatch (%lx < %lx)",
 				   uid,elt->private.uid);
 			  MM_LOG (tmp,ERROR);
 			  return NIL;
 			}
 		      }
+
 				/* time to create a new message? */
 		      if (nmsgs > stream->nmsgs) {
 				/* defer announcing until later */
@@ -1904,7 +1915,6 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 			elt->hours = hh; elt->minutes = mm;
 			elt->seconds = ss; elt->zoccident = z;
 			elt->zhours = zh; elt->zminutes = zm;
-
 				/* message in same file? */
 			if (curfile == file) {
 			  if (pos < curpos) {
@@ -1914,7 +1924,8 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 			    sprintf (tmp,shortmsg,plt->msgno,plt->private.uid,
 				     i,pos,curpos);
 				/* possible to fix? */
-			    if (!stream->rdonly && (i < plt->rfc822_size)) {
+			    if (!stream->rdonly && LOCAL->expok &&
+				(i < plt->rfc822_size)) {
 			      plt->rfc822_size -= i;
 			      if (plt->rfc822_size <
 				  plt->private.msg.header.text.size)
@@ -1948,7 +1959,8 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 			    sprintf (tmp,shortmsg,elt->msgno,elt->private.uid,
 				     i,curfilesize,curpos);
 				/* possible to fix? */
-			    if (!stream->rdonly && (i < elt->rfc822_size)) {
+			    if (!stream->rdonly && LOCAL->expok &&
+				(i < elt->rfc822_size)) {
 			      elt->rfc822_size -= i;
 			      if (elt->rfc822_size <
 				  elt->private.msg.header.text.size)
@@ -1994,97 +2006,101 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
       unsigned long uid,uf,sf,mod;
       char *s;
       int updatep = NIL;
-      int writep = (flags & (MP_VALID | MP_WRITE)) ? T : NIL;
 				/* open status file */
-      if ((fd = open (LOCAL->status,writep ? O_RDWR : O_RDONLY,NIL)) < 0)
+      if ((fd = open (LOCAL->status,
+		      stream->rdonly ? O_RDONLY : O_RDWR,NIL)) < 0)
 	MM_LOG ("Error opening mix status file",ERROR);
 				/* acquire exclusive access and FILE */
-      else if (!flock (fd,writep ? LOCK_EX : LOCK_SH) &&
-	       !(statf = fdopen (fd,writep ? "r+b" : "rb"))) {
+      else if (!flock (fd,stream->rdonly ? LOCK_SH : LOCK_EX) &&
+	       !(statf = fdopen (fd,stream->rdonly ? "rb" : "r+b"))) {
 	MM_LOG ("Error obtaining stream on mix status file",ERROR);
 	flock (fd,LOCK_UN);	/* relinquish lock */
 	close (fd);
       }
-      else if (!stream->nmsgs);	/* do nothing if mailbox empty */
 				/* get sequence */
-      else if (!(i = mix_read_sequence (statf)) || (i < LOCAL->statusseq)) {
+      else if (!(i = mix_read_sequence (statf)) ||
+	       ((i < LOCAL->statusseq) && stream->nmsgs && (i != 1))) {
 	sprintf (LOCAL->buf,
-		 "Error in mix status sequence record, i=%lu, seq=%lu",
+		 "Error in mix status sequence record, i=%lx, seq=%lx",
 		 i,LOCAL->statusseq);
 	MM_LOG (LOCAL->buf,ERROR);
       }
 				/* sequence changed from last time? */
-      else if (i > LOCAL->statusseq) {
-	LOCAL->statusseq = i;	/* update sequence, get first elt */
-	elt = mail_elt (stream,i = 1);
+      else if (i != LOCAL->statusseq) {
+				/* update sequence, get first elt */
+	if (i > LOCAL->statusseq) LOCAL->statusseq = i;
+	if (stream->nmsgs) {
+	  elt = mail_elt (stream,i = 1);
 
 				/* read message records */
-	while ((t = s = mix_read_record (statf,LOCAL->buf,LOCAL->buflen,
-					 "status")) && *s && (*s++ == ':') &&
-	       isxdigit (*s)) {
-	  uid = strtoul (s,&s,16);
-	  if ((*s++ == ':') && isxdigit (*s)) {
-	    uf = strtoul (s,&s,16);
+	  while ((t = s = mix_read_record (statf,LOCAL->buf,LOCAL->buflen,
+					   "status")) && *s && (*s++ == ':') &&
+		 isxdigit (*s)) {
+	    uid = strtoul (s,&s,16);
 	    if ((*s++ == ':') && isxdigit (*s)) {
-	      sf = strtoul (s,&s,16);
+	      uf = strtoul (s,&s,16);
 	      if ((*s++ == ':') && isxdigit (*s)) {
-		mod = strtoul (s,&s,16);
+		sf = strtoul (s,&s,16);
+		if ((*s++ == ':') && isxdigit (*s)) {
+		  mod = strtoul (s,&s,16);
 				/* ignore expansion values */
-		if (*s++ == ':') {
+		  if (*s++ == ':') {
 				/* need to move ahead to next elt? */
-		  while ((uid > elt->private.uid) && (i < stream->nmsgs))
-		    elt = mail_elt (stream,++i);
+		    while ((uid > elt->private.uid) && (i < stream->nmsgs))
+		      elt = mail_elt (stream,++i);
 				/* update elt if altered */
-		  if ((uid == elt->private.uid) &&
-		      (!elt->valid || (mod != elt->private.mod))) {
-		    elt->user_flags = uf;
-		    elt->private.mod = mod;
-		    elt->seen = (sf & fSEEN) ? T : NIL;
-		    elt->deleted = (sf & fDELETED) ? T : NIL;
-		    elt->flagged = (sf & fFLAGGED) ? T : NIL;
-		    elt->answered = (sf & fANSWERED) ? T : NIL;
-		    elt->draft = (sf & fDRAFT) ? T : NIL;
+		    if ((uid == elt->private.uid) &&
+			(!elt->valid || (mod != elt->private.mod))) {
+		      elt->user_flags = uf;
+		      elt->private.mod = mod;
+		      elt->seen = (sf & fSEEN) ? T : NIL;
+		      elt->deleted = (sf & fDELETED) ? T : NIL;
+		      elt->flagged = (sf & fFLAGGED) ? T : NIL;
+		      elt->answered = (sf & fANSWERED) ? T : NIL;
+		      elt->draft = (sf & fDRAFT) ? T : NIL;
 				/* announce if altered existing message */
-		    if (elt->valid) MM_FLAGS (stream,elt->msgno);
+		      if (elt->valid) MM_FLAGS (stream,elt->msgno);
 				/* first time, is old message? */
-		    else if (sf & fOLD) {
+		      else if (sf & fOLD) {
 				/* yes, clear recent and set valid */
-		      elt->recent = NIL;
-		      elt->valid = T;
-		    }
+			elt->recent = NIL;
+			elt->valid = T;
+		      }
 				/* recent, allowed to update its status? */
-		    else if (flags & MP_VALID) {
+		      else if (sflags) {
 				/* yes, set valid and check in status */
-		      elt->valid = T;
-		      elt->private.mod = mix_modseq (elt->private.mod);
-		      updatep = T;
+			elt->valid = T;
+			elt->private.mod = mix_modseq (elt->private.mod);
+			updatep = T;
+		      }
+		      /* leave valid unset and recent if sflags not set */
 		    }
-		    /* leave valid unset if recent and MP_VALID not set */
+		    continue;	/* everything looks good */
 		  }
-		  continue;	/* everything looks good */
 		}
 	      }
 	    }
+	    break;		/* error somewhere */
 	  }
-	  break;		/* error somewhere */
-	}
 
-				/* error seen, or update needed? */
-	if (!t || *t || updatep) {
 	  if (t && *t) {	/* non-null means bogus record */
 	    char msg[MAILTMPLEN];
 	    sprintf (msg,"Error in mix status file message record%s: %.80s",
-		     writep ? ", fixing" : "",t);
+		     stream->rdonly ? "" : ", fixing",t);
 	    MM_LOG (msg,WARN);
+				/* update it if not readonly */
+	    if (!stream->rdonly) updatep = T;
 	  }
-	  if (!stream->rdonly)	/* either way, update status file */
+	  if (updatep) {		/* need to update? */
 	    LOCAL->statusseq = mix_modseq (LOCAL->statusseq);
-	  if (writep) mix_status_update (stream,&statf,NIL);
+	    mix_status_update (stream,statf);
+	  }
 	}
       }
     }
   }
   if (statf) {			/* still happy? */
+    unsigned long j;
     stream->silent = silent;	/* now notify upper level */
     mail_exists (stream,stream->nmsgs);
     for (i = 1, j = 0; i <= stream->nmsgs; ++i)
@@ -2179,7 +2195,7 @@ long mix_meta_update (MAILSTREAM *stream)
 /* MIX index file routines */
 
 
-/* MIX update and close index
+/* MIX update index
  * Accepts: MAIL stream
  *	    open FILE
  * Returns: T on success, NIL if error
@@ -2222,52 +2238,43 @@ long mix_index_update (MAILSTREAM *stream,FILE *idxf)
 /* MIX status file routines */
 
 
-/* MIX update and close status
+/* MIX update status
  * Accepts: MAIL stream
  *	    pointer to open FILE
- *	    close when done flag
  * Returns: T on success, NIL if error
  */
 
-long mix_status_update (MAILSTREAM *stream,FILE **statf,long flag)
+long mix_status_update (MAILSTREAM *stream,FILE *statf)
 {
-  FILE *f = *statf;
   unsigned long i;
   char tmp[MAILTMPLEN];
   long ret = LONGT;
   if (!stream->rdonly) {	/* do nothing if stream readonly */
-    rewind (f);			/* let's start at the very beginning */
+    rewind (statf);		/* let's start at the very beginning */
 				/* write sequence */
-    fprintf (f,SEQFMT,LOCAL->statusseq);
+    fprintf (statf,SEQFMT,LOCAL->statusseq);
 				/* write message status records */
     for (i = 1; ret && (i <= stream->nmsgs); ++i) {
       MESSAGECACHE *elt = mail_elt (stream,i);
 				/* make sure all messages have a modseq */
       if (!elt->private.mod) elt->private.mod = LOCAL->statusseq;
-      fprintf (f,STRFMT,elt->private.uid,elt->user_flags,
-	       (fSEEN * elt->seen) + (fDELETED * elt->deleted) +
-	       (fFLAGGED * elt->flagged) + (fANSWERED * elt->answered) +
-	       (fDRAFT * elt->draft) + (elt->valid ? fOLD : NIL),
-	       elt->private.mod);
-      if (ferror (f)) {
+      if (!elt->private.ghost)	/* only write living messages */
+	fprintf (statf,STRFMT,elt->private.uid,elt->user_flags,
+		 (fSEEN * elt->seen) + (fDELETED * elt->deleted) +
+		 (fFLAGGED * elt->flagged) + (fANSWERED * elt->answered) +
+		 (fDRAFT * elt->draft) + (elt->valid ? fOLD : NIL),
+		 elt->private.mod);
+      if (ferror (statf)) {
 	sprintf (tmp,"Error updating mix status file: %.80s",strerror (errno));
 	MM_LOG (tmp,ERROR);
 	ret = NIL;
       }
     }
-    if (ret && fflush (f)) {
+    if (ret && fflush (statf)) {
       MM_LOG ("Error flushing mix status file",ERROR);
       ret = NIL;
     }
-    if (ret) ftruncate (fileno (f),ftell (f));
-  }
-  if (flag) {			/* close index if requested */
-    if (fclose (f)) {
-      sprintf (tmp,"Error closing mix status file: %.80s",strerror (errno));
-      MM_LOG (tmp,ERROR);
-      ret = NIL;
-    }
-    *statf = NIL;		/* either way, status is closed */
+    if (ret) ftruncate (fileno (statf),ftell (statf));
   }
   return ret;
 }
@@ -2296,8 +2303,9 @@ FILE *mix_data_open (MAILSTREAM *stream,int *fd,long *size,
   unsigned long curend = (elt && (elt->private.spare.data == LOCAL->newmsg)) ?
     elt->private.special.offset + elt->private.msg.header.offset +
     elt->rfc822_size : 0;
+				/* allow create if curend 0 */
   if ((*fd = open (mix_file_data (LOCAL->buf,stream->mailbox,LOCAL->newmsg),
-		   O_RDWR,NIL)) >= 0) {
+		   O_RDWR | (curend ? NIL : O_CREAT),NIL)) >= 0) {
     fstat (*fd,&sbuf);		/* get current file size */
 				/* can we use this file? */
     if ((curend <= sbuf.st_size) &&
@@ -2306,7 +2314,7 @@ FILE *mix_data_open (MAILSTREAM *stream,int *fd,long *size,
     else {			/* short file or becoming too long */
       if (curend > sbuf.st_size) {
 	char tmp[MAILTMPLEN];
-	sprintf (tmp,"short mix message file %lx (%ld > %ld), rolling",
+	sprintf (tmp,"short mix message file %.08lx (%ld > %ld), rolling",
 		 LOCAL->newmsg,curend,sbuf.st_size);
 	MM_LOG (tmp,WARN);	/* shouldn't happen */
       }
@@ -2645,7 +2653,7 @@ char *mix_read_record (FILE *f,char *buf,unsigned long buflen,char *type)
  * Returns: sequence value, or NIL if failure
  */
 
-long mix_read_sequence (FILE *f)
+unsigned long mix_read_sequence (FILE *f)
 {
   unsigned long ret;
   char *s,tmp[MAILTMPLEN];
