@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: pilot.c 672 2007-08-15 23:07:18Z hubert@u.washington.edu $";
+static char rcsid[] = "$Id: pilot.c 737 2007-10-03 19:40:34Z hubert@u.washington.edu $";
 #endif
 
 /*
@@ -70,19 +70,21 @@ N_("\t -x \t\tNoKeyhelp - suppress keyhelp"),
 N_("\t -q \t\tTermdefWins - termcap or terminfo takes precedence over defaults"),
 N_("\t -f \t\tKeys - force use of function keys"),
 N_("\t -h \t\tHelp - give this list of options"),
+#ifndef	_WINDOWS
 N_("\t -dcs <display_character_set> \tdefault uses LANG or LC_CTYPE from environment"),
 N_("\t -kcs <keyboard_character_set> \tdefaults to display_character_set"),
 N_("\t -syscs\t\tuse system-supplied translation routines"),
+#endif	/* ! _WINDOWS */
 N_("\t -n[#s] \tMail - notify about new mail every #s seconds, default=180"),
 N_("\t -t \t\tShutdown - enable special shutdown mode"),
 N_("\t -o <dir>\tOperation - specify the operating directory"),
 N_("\t -z \t\tSuspend - allow use of ^Z suspension"),
-#if     defined(DOS) || defined(OS2)
+#ifdef	_WINDOWS
 N_("\t -cnf color \tforeground color"),
 N_("\t -cnb color \tbackground color"),
 N_("\t -crf color \treverse foreground color"),
 N_("\t -crb color \treverse background color"),
-#endif
+#endif	/* _WINDOWS */
 N_("\t -no_setlocale_collate\tdo not do setlocale(LC_COLLATE)"),
 "", 
 N_("\t All arguments may be followed by a directory name to start in."),
@@ -130,6 +132,10 @@ main(int argc, char *argv[])
 
     set_collation(setlocale_collate, 1);
 
+#ifdef	_WINDOWS
+    init_utf8_display(1, NULL);
+#else	/* UNIX */
+
 #define cpstr(s) strcpy((char *)fs_get(1+strlen(s)), s)
 
     if(display_character_set)
@@ -162,8 +168,6 @@ main(int argc, char *argv[])
 	    fprintf(stderr, "%s\n", err);
 	    fs_give((void **) &err);
 	}
-#elif	_WINDOWS	
-	fprintf(stderr, "%s\n", _("Option -syscs ignored due to missing system functionality"));
 #endif
     }
 
@@ -187,6 +191,8 @@ main(int argc, char *argv[])
     if(display_charmap)
       free((void *) display_charmap);
 
+#endif	/* UNIX */
+
     if(!vtinit())			/* Displays.            */
       exit(1);
 
@@ -244,11 +250,13 @@ Loop:
 	    *setlocale_collate = 0;
 	    goto Loop;
 	}
+#ifndef	_WINDOWS
 	else if(strcmp(*av, "syscs") == 0){
 	    use_system_translation = !use_system_translation;
 	    goto Loop;
 	}
-#if	defined(DOS) || defined(OS2)
+#endif	/* ! _WINDOWS */
+#ifdef	_WINDOWS
 	if(strcmp(*av, "cnf") == 0
 	   || strcmp(*av, "cnb") == 0
 	   || strcmp(*av, "crf") == 0
@@ -279,7 +287,8 @@ Loop:
 
 	    goto Loop;
 	}
-#endif
+#endif	/* _WINDOWS */
+#ifndef	_WINDOWS
 	else if(strcmp(*av, "dcs") == 0 || strcmp(*av, "kcs") == 0){
 	    char *cmd = *av;
 
@@ -309,6 +318,7 @@ Loop:
 
 	    goto Loop;
 	}
+#endif	/* ! _WINDOWS */
 
 	/*
 	 * Single char options.
