@@ -1,8 +1,8 @@
 /*
- * $Id: mimetype.h 136 2006-09-22 20:06:05Z hubert@u.washington.edu $
+ * $Id: mimetype.h 701 2007-08-31 18:52:30Z hubert@u.washington.edu $
  *
  * ========================================================================
- * Copyright 2006 University of Washington
+ * Copyright 2006-2007 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -35,10 +35,16 @@ typedef struct {
 } MT_MAP_T;
 
 
+typedef int (* MT_OPERATORPROC)(MT_MAP_T *, FILE *);
+
+
 /* exported protoypes */
 int	    set_mime_type_by_extension(BODY *, char *);
 int	    set_mime_extension_by_type(char *, char *);
 int         mt_srch_by_ext(MT_MAP_T *, FILE *);
+int         mt_get_file_ext(char *, char **);
+int         mt_srch_mime_type(MT_OPERATORPROC, MT_MAP_T *);
+int         mt_translate_type(char *);
 
 
 #endif /* PITH_MIMETYPE_INCLUDED */
