@@ -1,7 +1,7 @@
 # Web Alpine Config options
 # $Id$
 # ========================================================================
-# Copyright 2006 University of Washington
+# Copyright 2006-2008 University of Washington
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,31 +12,28 @@
 # ========================================================================
 
 set _wp(appname)	Alpine
-set _wp(version)	0.80
+set _wp(version)	1.00
 set _wp(admin)		admin@sample-domain.edu
 set _wp(helpdesk)	admin@sample-domain.edu
 set _wp(comments)	help@sample-domain.edu
 
+# List of userid's allowed to request the monitor script output
 set _wp(monitors)	{}
 
-set _wp(fileroot)	/var/www/alpine
+# directory prefix web server uses for web alpine page requests
+# Note: set to {} if DocumentRoot set to the root of web alpine cgi scripts
+set _wp(urlprefix)	webmail
+
+# file system path to CGI application files
+# directory containing web alpine application scripts and supporting tools
+set _wp(fileroot)	/usr/local/libexec/alpine
 
 set _wp(tmpdir)		/tmp
 
-# file system path to CGI application files on this server
-set _wp(cgiroot)	[file join $_wp(fileroot) cgi]
-
-# any subdirectories you add between $_wp(cgiroot) (typically
-# the same as httpd's DocumentRoot) and web alpine's CGI script
-# directory tree
-set _wp(cgidir)		{}
-
-# filesystem root of CGI application scripts.
 # NOTE: Make SURE tclsh and alpine.tcl symlinks in this directory
-#       point to their $_wp(fileroot) targets
-set _wp(cgipath)	[file join $_wp(cgiroot) $_wp(cgidir)]
+set _wp(cgipath)	[file join $_wp(fileroot) cgi]
 
-# place for web alpine application scripts
+# place for CGI scripts that execute the interface
 set _wp(appdir)		alpine
 
 # place for CGI scripts not requiring session-key
@@ -59,7 +56,7 @@ set _wp(lib)		[file join $_wp(fileroot) lib]
 # directory used temporarily to stage attatched and detached files
 set _wp(detachpath)	[file join $_wp(fileroot) detach]
 
-set _wp(imagepath)	[file join / $_wp(cgidir) images]
+set _wp(imagepath)	[file join / $_wp(urlprefix) images]
 
 set _wp(buttonpath)	[file join $_wp(imagepath) buttons silver]
 
@@ -68,15 +65,15 @@ set _wp(staticondir)	env
 set _wp(servername)	[info hostname]
 
 # MUST specify SSL/TLS connection
-set _wp(serverport)	5131
-set _wp(serverpath)	https://[file join [join [eval list $_wp(servername) $_wp(serverport)] :] $_wp(cgidir)]
+set _wp(serverport)	{}
+set _wp(serverpath)	https://[file join [join [eval list $_wp(servername) $_wp(serverport)] :] $_wp(urlprefix)]
 
 # MAY specify a plaintext connection (comment out if plain support undesired)
-set _wp(plainport)	5132
-set _wp(plainservpath)	http://[file join [join [eval list $_wp(servername) $_wp(plainport)] :] $_wp(cgidir)]
+set _wp(plainport)	{}
+set _wp(plainservpath)	http://[file join [join [eval list $_wp(servername) $_wp(plainport)] :] $_wp(urlprefix)]
 
 # url of faq page(s) available from initial greeting page
-#set _wp(faq)		"http://www.washington.edu/computing/faqs/webpine.html"
+#set _wp(faq)		"http://www.yourserver/faqs/alpine.html"
 
 # url of informational page accessible from initial greeting page
 set _wp(releaseblurb)	"$_wp(plainservpath)/alpine/help/release.html"
@@ -112,7 +109,8 @@ set _wp(titlesep)	4
 set _wp(config)		remote_pinerc
 set _wp(motd)		motd
 set _wp(save_cache_max)	6
-set _wp(fldr_cache_max)	3
+set _wp(fldr_cache_max)	8
+set _wp(fldr_cache_def)	3
 set _wp(statushelp)	0
 set _wp(imgbuttons)	0
 set _wp(keybindings)	1
@@ -153,7 +151,6 @@ set _wp(indexheight) [expr {$_wp(indexheight) <= 20 ? 20 : $_wp(indexheight) >=
 #set _wp(spamfolder)	junk-mail
 #set _wp(spamsubj)	"ATTACHED SPAM"
 
-
 #
 # Nickname server bindings.  If not present, prompt for the
 # destination of the default pinerc location.
@@ -164,17 +161,9 @@ set _wp(hosts) {
       $User.deskmail.washington.edu/ssl
       $_wp(confdir)/conf.deskmail
     }
-    {
-      NDC-MS
-      ndcms.cac.washington.edu/ssl
-      $_wp(confdir)/conf.ndcms
-    }
 }
 
-
-#
-# Everbody inherits the cgi package
-#
+# Everybody inherits the cgi, comm packages
 lappend auto_path $_wp(lib)
 
 package require cgi
@@ -356,42 +345,9 @@ proc WPEval {vars cmd} {
 	    cgi_debug -on
 	}
 
-	# Import data and validate it
-	if {[catch {cgi_input "sessid=8543949466398&"} result]} {
-	  WPInfoPage "Web Alpine Error" [font size=+2 $result] "Please close this window."
-	  return
-	}
-
 	# Session id?
-	if {[catch {WPImport sessid "Missing Session ID"} errstr]} {
-	  if {[regexp {.*sessid.*no such.*} $errstr]} {
-	    WPInactivePage [list "Your browser may have failed to send the necessary <i>cookie</i> information.  Please verify your browser configuration has cookies enabled."]
-	  } else {
-	    WPInfoPage "Web Alpine Error" [font size=+2 $errstr] "Please close this window."
-	  }
-
+	if {[catch {WPGetInputAndID sessid}]} {
 	  return
-	} else {
-	    # initialization here
-	    if {[catch {WPValidId $sessid} result]} {
-	      if {[string compare [lindex $result 0] redirect]} {
-		WPInfoPage "Web Alpine Error" [font size=+2 "$result"] \
-			"Please complain to the [cgi_link Admin] and visit the [cgi_link Start] later."
-	      } else {
-		cgi_http_head {
-		  cgi_redirect [lindex $result 1]
-		}
-	      }
-
-	      return
-	    } elseif {$_wp(sessid) == 0} {
-	      WPInactivePage
-	      return
-	    }
-
-	    if {[catch {WPCmd set serverroot} serverroot] == 0} {
-	      cgi_root $serverroot
-	    }
 	}
 
 	foreach item $_wp(vars) {
@@ -474,6 +430,49 @@ proc WPEval {vars cmd} {
     }
 }
 
+proc WPGetInputAndID {_sessid} {
+  global _wp
+  upvar $_sessid sessid
+
+  # Import data and validate it
+  if {[catch {cgi_input "sessid=8543949466398&"} result]} {
+    WPInfoPage "Web Alpine Error" [font size=+2 $result] "Please close this window."
+    error "Cannot get CGI Input"
+  }
+
+  if {[catch {WPImport sessid "Missing Session ID"} errstr]} {
+    if {[regexp {.*sessid.*no such.*} $errstr]} {
+      WPInactivePage [list "Your browser may have failed to send the necessary <i>cookie</i> information.  Please verify your browser configuration has cookies enabled."]
+    } else {
+      WPInfoPage "Web Alpine Error" [font size=+2 $errstr] "Please close this window."
+    }
+
+    error "Session ID Failure"
+  } else {
+    # initialization here
+    if {[catch {WPValidId $sessid} result]} {
+      if {[string compare [lindex $result 0] redirect]} {
+	WPInfoPage "Web Alpine Error" [font size=+2 "$result"] \
+	    "Please complain to the [cgi_link Admin] and visit the [cgi_link Start] later."
+      } else {
+	cgi_http_head {
+	  cgi_redirect [lindex $result 1]
+	}
+      }
+
+      error "Unrecoverable Error"
+    } elseif {$_wp(sessid) == 0} {
+      WPInactivePage
+      error "Inactive Session"
+    }
+
+    if {[catch {WPCmd set serverroot} serverroot] == 0} {
+      cgi_root $serverroot
+    }
+  }
+
+}
+
 proc WPCmd {args} {
   global _wp
 
@@ -531,7 +530,7 @@ proc WPImport {valname {errstring ""} {default 0}} {
 proc WPExportCookie {name value {scope ""}} {
     global _wp
 
-    cgi_cookie_set $name=$value "path=[file join / $_wp(cgidir) $scope]"
+    cgi_cookie_set $name=$value "path=[file join / $_wp(urlprefix) $scope]"
 }
 
 
@@ -562,7 +561,7 @@ proc WPInfoPage {title exp1 {exp2 ""} {imgurl {}} {exp3 ""}} {
     cgi_html {
       cgi_head {
 	cgi_title $title
-	cgi_stylesheet [file join / $_wp(cgidir) $_wp(pubdir) standard.css]
+	cgi_stylesheet [file join / $_wp(urlprefix) $_wp(pubdir) standard.css]
       }
 
       cgi_body {
@@ -699,7 +698,7 @@ proc WPIndexLineHeight {{ih 0}} {
 proc WPStyleSheets {{ih 0}} {
   global _wp
 
-  cgi_stylesheet [file join / $_wp(cgidir) $_wp(pubdir) standard.css]
+  cgi_stylesheet [file join / $_wp(urlprefix) $_wp(pubdir) standard.css]
 
   if {$ih <= 0} {
     set ih [WPIndexLineHeight]
@@ -714,7 +713,7 @@ proc WPStdScripts {{ih 0}} {
 
     set ih [WPStyleSheets $ih]
 
-    cgi_script language="JavaScript" src="[file join / $_wp(cgidir) $_wp(pubdir) standard.js]" {}
+    cgi_script language="JavaScript" src="[file join / $_wp(urlprefix) $_wp(pubdir) standard.js]" {}
     cgi_script language="JavaScript1.3" {cgi_put "js_version = '1.3';"}
     cgi_javascript {
 	cgi_puts "function getIndexHeight(){return $ih}"
@@ -832,12 +831,9 @@ proc WPNewMail {reload {viewpage msgview.tcl}} {
 	lappend newref [list $newtext "" ""]
     }
 
-    set statmsgs [WPCmd PEInfo statmsgs]
-    foreach statmsg $statmsgs {
-	if {[string length $statmsg] > 0} {
-	    lappend newref [list $statmsg "" ""]
-	    WPCmd PEInfo statmsg ""
-	}
+    foreach statmsg [WPStatusMsgs] {
+      lappend newref [list $statmsg "" ""]
+      WPCmd PEInfo statmsg ""
     }
 
     if {!$reload} {
@@ -847,6 +843,26 @@ proc WPNewMail {reload {viewpage msgview.tcl}} {
     return $newref
 }
 
+proc WPStatusMsgs {} {
+  set retmsgs ""
+  set lastmsg ""
+  if {[catch {WPCmd PEInfo statmsgs} statmsgs] == 0} {
+    foreach statmsg $statmsgs {
+      if {[string length $statmsg] > 0 && [string compare $statmsg $lastmsg]} {
+	if {[regexp "^Pinerc \(.+\) NOT saved$" $statmsg]} {
+	  lappend retmsgs "Another Pine/WebPine session may be running.  Settings cannot be saved."
+	} else {
+	  lappend retmsgs $statmsg
+	}
+
+	set lastmsg $statmsg
+      }
+    }
+  }
+
+  return $retmsgs
+}
+
 proc WPStatusIcon {uid {extension gif} {statbits ""}} {
   global _wp
 
